@@ -9,7 +9,7 @@
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; under version 2 of the License.
+ *  the Free Software Foundation; under version 3 of the License.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -17,8 +17,10 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *  along with this program.  If not, see <http://www.gnu.org/licenses>.
+ *
+ *  The Audacious team does not consider modular code linking to
+ *  Audacious or using our public API to be a derived work.
  */
 
 #ifndef EFFECT_H
@@ -26,6 +28,9 @@
 
 #include <glib.h>
 
+#include "audacious/flow.h"
+#include "audacious/output.h"
+
 typedef struct _EffectPluginData EffectPluginData;
 
 struct _EffectPluginData {
@@ -34,13 +39,14 @@ struct _EffectPluginData {
 };
 
 GList *get_effect_list(void);
-void effect_about(gint i);
-void effect_configure(gint i);
 GList *get_effect_enabled_list(void);
 void enable_effect_plugin(gint i, gboolean enable);
-gboolean effect_enabled(gint i);
 gchar *effect_stringify_enabled_list(void);
 void effect_enable_from_stringified_list(const gchar * list);
+gint effect_do_mod_samples(gpointer * data, gint length, AFormat fmt, 
+	gint srate, gint nch);
+void effect_do_query_format(AFormat * fmt, gint * rate, gint * nch);
+void effect_flow(FlowContext *context);
 
 extern EffectPluginData ep_data;
 
