@@ -9,7 +9,7 @@
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; under version 2 of the License.
+ *  the Free Software Foundation; under version 3 of the License.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -17,8 +17,10 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *  along with this program.  If not, see <http://www.gnu.org/licenses>.
+ *
+ *  The Audacious team does not consider modular code linking to
+ *  Audacious or using our public API to be a derived work.
  */
 #ifndef PLUGINENUM_H
 #define PLUGINENUM_H
@@ -34,4 +36,10 @@ void plugin_system_cleanup(void);
 extern const gchar *plugin_dir_list[];
 extern GHashTable *plugin_matrix;
 
+gboolean plugin_is_enabled(const gchar *filename);
+void plugin_set_enabled(const gchar *filename, gboolean enabled);
+Plugin *plugin_get_plugin(const gchar *filename);
+
+void plugin_set_current(Plugin *plugin);
+
 #endif
