@@ -1,5 +1,5 @@
 /*  Audacious - Cross-platform multimedia player
- *  Copyright (C) 2005-2007  Audacious development team
+ *  Copyright (C) 2005-2008  Audacious development team
  *
  *  Based on BMP:
  *  Copyright (C) 2003-2004  BMP development team.
@@ -9,7 +9,7 @@
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; under version 2 of the License.
+ *  the Free Software Foundation; under version 3 of the License.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -17,41 +17,46 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *  along with this program.  If not, see <http://www.gnu.org/licenses>.
+ *
+ *  The Audacious team does not consider modular code linking to
+ *  Audacious or using our public API to be a derived work.
  */
 
+/*#define AUD_DEBUG*/
+
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
 #endif
 
-#define NEED_GLADE
 #include "util.h"
 
 #include <glib.h>
 #include <glib/gi18n.h>
-#include <glade/glade.h>
 #include <gtk/gtk.h>
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
+#include <math.h>
 
 #include "platform/smartinclude.h"
 #include <errno.h>
 
 #ifdef HAVE_FTS_H
+#  include <sys/types.h>
+#  include <sys/stat.h>
 #  include <fts.h>
 #endif
 
-#include "glade.h"
 #include "input.h"
 #include "main.h"
 #include "playback.h"
 #include "strings.h"
 #include "ui_playlist.h"
+#include "libSAD.h"
 
 #ifdef USE_CHARDET
-    #include "../librcd/librcd.h"
+    #include "../libguess/libguess.h"
 #ifdef HAVE_UDET
     #include <libudet_c.h>
 #endif
@@ -69,24 +74,71 @@ typedef struct {
     gboolean found;
 } FindFileContext;
 
+static const struct {
+    AFormat afmt;
+    SAD_sample_format sadfmt;
+} format_table[] = {
+    {FMT_U8, SAD_SAMPLE_U8},
+    {FMT_S8, SAD_SAMPLE_S8},
+
+    {FMT_S16_LE, SAD_SAMPLE_S16_LE},
+    {FMT_S16_BE, SAD_SAMPLE_S16_BE},
+    {FMT_S16_NE, SAD_SAMPLE_S16},
+
+    {FMT_U16_LE, SAD_SAMPLE_U16_LE},
+    {FMT_U16_BE, SAD_SAMPLE_U16_BE},
+    {FMT_U16_NE, SAD_SAMPLE_U16},
+
+    {FMT_S24_LE, SAD_SAMPLE_S24_LE},
+    {FMT_S24_BE, SAD_SAMPLE_S24_BE},
+    {FMT_S24_NE, SAD_SAMPLE_S24},
+    
+    {FMT_U24_LE, SAD_SAMPLE_U24_LE},
+    {FMT_U24_BE, SAD_SAMPLE_U24_BE},
+    {FMT_U24_NE, SAD_SAMPLE_U24},
+
+    {FMT_S32_LE, SAD_SAMPLE_S32_LE},
+    {FMT_S32_BE, SAD_SAMPLE_S32_BE},
+    {FMT_S32_NE, SAD_SAMPLE_S32},
+    
+    {FMT_U32_LE, SAD_SAMPLE_U32_LE},
+    {FMT_U32_BE, SAD_SAMPLE_U32_BE},
+    {FMT_U32_NE, SAD_SAMPLE_U32},
+    
+    {FMT_FLOAT, SAD_SAMPLE_FLOAT},
+    {FMT_FIXED32, SAD_SAMPLE_FIXED32},
+};
+
+SAD_sample_format
+sadfmt_from_afmt(AFormat fmt)
+{
+    int i;
+    for (i = 0; i < sizeof(format_table) / sizeof(format_table[0]); i++) {
+        if (format_table[i].afmt == fmt) return format_table[i].sadfmt;
+    }
+
+    return -1;
+}
+
+
 static gboolean
 find_file_func(const gchar * path, const gchar * basename, gpointer data)
 {
     FindFileContext *context = data;
 
     if (strlen(path) > FILENAME_MAX) {
-        g_warning("Ignoring path: name too long (%s)", path);
+        AUDDBG("Ignoring path: name too long (%s)\n", path);
         return TRUE;
     }
 
-    if (g_file_test(path, G_FILE_TEST_IS_REGULAR)) {
+    if (vfs_file_test(path, G_FILE_TEST_IS_REGULAR)) {
         if (!strcasecmp(basename, context->to_match)) {
             context->match = g_strdup(path);
             context->found = TRUE;
             return TRUE;
         }
     }
-    else if (g_file_test(path, G_FILE_TEST_IS_DIR)) {
+    else if (vfs_file_test(path, G_FILE_TEST_IS_DIR)) {
         dir_foreach(path, find_file_func, context, NULL);
         if (context->found)
             return TRUE;
@@ -99,12 +151,34 @@ gchar *
 find_file_recursively(const gchar * path, const gchar * filename)
 {
     FindFileContext context;
+    gchar *out = NULL;
 
     context.to_match = filename;
     context.match = NULL;
     context.found = FALSE;
 
     dir_foreach(path, find_file_func, &context, NULL);
+
+    if (context.match)
+    {
+        out = g_filename_to_uri(context.match, NULL, NULL);
+        g_free(context.match);
+    }
+
+    return out;
+}
+
+gchar *
+find_path_recursively(const gchar * path, const gchar * filename)
+{
+    FindFileContext context;
+
+    context.to_match = filename;
+    context.match = NULL;
+    context.found = FALSE;
+
+    dir_foreach(path, find_file_func, &context, NULL);
+
     return context.match;
 }
 
@@ -277,12 +351,12 @@ archive_decompress(const gchar * filename)
     tmpdir = g_build_filename(g_get_tmp_dir(), "audacious.XXXXXXXX", NULL);
     if (!mkdtemp(tmpdir)) {
         g_free(tmpdir);
-        g_message("Unable to load skin: Failed to create temporary "
-                  "directory: %s", g_strerror(errno));
+        AUDDBG("Unable to load skin: Failed to create temporary "
+                  "directory: %s\n", g_strerror(errno));
         return NULL;
     }
 #else
-    tmpdir = g_strdup_printf("%s/audacious.%ld", g_get_tmp_dir(), rand());
+    tmpdir = g_strdup_printf("%s/audacious.%ld", g_get_tmp_dir(), (long) rand());
     make_directory(tmpdir, mode755);
 #endif
 
@@ -291,14 +365,16 @@ archive_decompress(const gchar * filename)
     g_free(escaped_filename);
 
     if (!cmd) {
-        g_message("extraction function is NULL!");
+        AUDDBG("extraction function is NULL!\n");
         g_free(tmpdir);
         return NULL;
     }
 
-    if(system(cmd) == -1)
+    AUDDBG("Attempt to execute \"%s\"\n", cmd);
+
+    if(system(cmd) != 0)
     {
-        g_message("could not execute cmd %s",cmd);
+        AUDDBG("could not execute cmd %s\n",cmd);
         g_free(cmd);
         return NULL;
     }
@@ -378,18 +454,31 @@ strip_string(GString *string)
 static void
 strip_lower_string(GString *string)
 {
+    gchar *lower;
     strip_string(string);
 
-    gchar *lower = g_ascii_strdown(string->str, -1);
+    lower = g_ascii_strdown(string->str, -1);
     g_free(string->str);
     string->str = lower;
 }
 
+static void
+close_ini_file_free_value(gpointer value)
+{
+    g_free((gchar*)value);
+}
+
+static void
+close_ini_file_free_section(gpointer section)
+{
+    g_hash_table_destroy((GHashTable*)section);
+}
+
 INIFile *
 open_ini_file(const gchar *filename)
 {
-    GHashTable *ini_file = g_hash_table_new(NULL, NULL);
-    GHashTable *section = g_hash_table_new(NULL, NULL);
+    GHashTable *ini_file = NULL;
+    GHashTable *section = NULL;
     GString *section_name, *key_name, *value;
     gpointer section_hash, key_hash;
     gchar *buffer = NULL;
@@ -398,23 +487,11 @@ open_ini_file(const gchar *filename)
 
     unsigned char x[] = { 0xff, 0xfe, 0x00 };
 
-
     g_return_val_if_fail(filename, NULL);
-
-    section_name = g_string_new("");
-    key_name = g_string_new(NULL);
-    value = g_string_new(NULL);
-
-    /* make a nameless section which should store all entries that are not
-     * embedded in a section */
-    section_hash = GINT_TO_POINTER(g_string_hash(section_name));
-    g_hash_table_insert(ini_file, section_hash, section);
-
     vfs_file_get_contents(filename, &buffer, &filesize);
     if (buffer == NULL)
         return NULL;
 
-
     /*
      * Convert UTF-16 into something useful. Original implementation
      * by incomp@#audacious. Cleanups \nenolod
@@ -427,10 +504,13 @@ open_ini_file(const gchar *filename)
 
         for (counter = 2; counter < filesize; counter += 2)
         {
-            if (!memcmp(&buffer[counter+1], &x[2], 1))
+            if (!memcmp(&buffer[counter+1], &x[2], 1)) {
                 outbuf[(counter-2)/2] = buffer[counter];
-            else
+            } else {
+                g_free(buffer);
+                g_free(outbuf);
                 return NULL;
+            }
         }
 
         outbuf[(counter-2)/2] = '\0';
@@ -442,11 +522,25 @@ open_ini_file(const gchar *filename)
         }
         else
         {
+            g_free(buffer);
             g_free(outbuf);
             return NULL;    /* XXX wrong encoding */
         }
     }
 
+    section_name = g_string_new("");
+    key_name = g_string_new(NULL);
+    value = g_string_new(NULL);
+
+    ini_file = g_hash_table_new_full(NULL, NULL, NULL,
+                                     close_ini_file_free_section);
+    section = g_hash_table_new_full(NULL, NULL, NULL,
+                                    close_ini_file_free_value);
+    /* make a nameless section which should store all entries that are not
+     * embedded in a section */
+    section_hash = GINT_TO_POINTER(g_string_hash(section_name));
+    g_hash_table_insert(ini_file, section_hash, section);
+
     while (off < filesize)
     {
         /* ignore the following characters */
@@ -456,7 +550,7 @@ open_ini_file(const gchar *filename)
             if (buffer[off] == '\n')
             {
                 g_string_free(key_name, TRUE);
-                g_string_free(value, FALSE);
+                g_string_free(value, TRUE);
                 key_name = g_string_new(NULL);
                 value = g_string_new(NULL);
             }
@@ -504,7 +598,8 @@ open_ini_file(const gchar *filename)
                     section = g_hash_table_lookup(ini_file, section_hash);
                 else
                 {
-                    section = g_hash_table_new(NULL, NULL);
+                    section = g_hash_table_new_full(NULL, NULL, NULL,
+                                                    close_ini_file_free_value);
                     g_hash_table_insert(ini_file, section_hash, section);
                 }
 
@@ -550,38 +645,53 @@ return_sequence:
     return ini_file;
 }
 
+/**
+ * Frees the memory allocated for inifile.
+ */
 void
 close_ini_file(INIFile *inifile)
 {
     g_return_if_fail(inifile);
-
-    /* we don't have to destroy anything in the hash table manually, as the
-     * keys are represented as integers and the string values may be used in
-     * functions which have read the strings from the hash table
-     */
     g_hash_table_destroy(inifile);
 }
 
+/**
+ * Returns a string that corresponds to correct section and key in inifile.
+ *
+ * Returns NULL if value was not found in inifile. Otherwise returns a copy
+ * of string pointed by "section" and "key". Returned string should be freed
+ * after use.
+ */
 gchar *
 read_ini_string(INIFile *inifile, const gchar *section, const gchar *key)
 {
+    GString *section_string;
+    GString *key_string;
+    gchar *value = NULL;
+    gpointer section_hash, key_hash;
+    GHashTable *section_table;
+    
     g_return_val_if_fail(inifile, NULL);
 
-    GString *section_string = g_string_new(section);
-    GString *key_string = g_string_new(key);
-    gchar *value = NULL;
+    section_string = g_string_new(section);
+    key_string = g_string_new(key);
+    value = NULL;
 
     strip_lower_string(section_string);
     strip_lower_string(key_string);
-    gpointer section_hash = GINT_TO_POINTER(g_string_hash(section_string));
-    gpointer key_hash = GINT_TO_POINTER(g_string_hash(key_string));
-    g_string_free(section_string, FALSE);
-    g_string_free(key_string, FALSE);
+    section_hash = GINT_TO_POINTER(g_string_hash(section_string));
+    key_hash = GINT_TO_POINTER(g_string_hash(key_string));
+    section_table = g_hash_table_lookup(inifile, section_hash);
 
-    GHashTable *section_table = g_hash_table_lookup(inifile, section_hash);
-    g_return_val_if_fail(section_table, NULL);
+    if (section_table) {
+        value = g_strdup(g_hash_table_lookup(section_table,
+                                             GINT_TO_POINTER(key_hash)));
+    }
 
-    value = g_hash_table_lookup(section_table, GINT_TO_POINTER(key_hash));
+    g_string_free(section_string, TRUE);
+    g_string_free(key_string, TRUE);
+
+    g_return_val_if_fail(value, NULL);
     return value;
 }
 
@@ -800,182 +910,17 @@ make_filebrowser(const gchar *title, gboolean save)
     return dialog;
 }
 
-/*
- * Resizes a GDK pixmap.
- */
-GdkPixmap *audacious_pixmap_resize(GdkWindow *src, GdkGC *src_gc, GdkPixmap *in, gint width, gint height)
-{
-    GdkPixmap *out;
-    gint owidth, oheight;
+void ui_skinned_widget_draw(GtkWidget *widget, GdkPixbuf *obj, gint width, gint height, gboolean scale) {
+    g_return_if_fail(widget != NULL);
+    g_return_if_fail(obj != NULL);
 
-    g_return_val_if_fail(src != NULL, NULL);
-    g_return_val_if_fail(src_gc != NULL, NULL);
-    g_return_val_if_fail(in != NULL, NULL);
-    g_return_val_if_fail(width > 0 && height > 0, NULL);
-
-    gdk_drawable_get_size(in, &owidth, &oheight);
-
-    if (oheight == height && owidth == width)
-        return NULL;
-
-    out = gdk_pixmap_new(src, width, height, -1);
-
-    gdk_draw_rectangle(out, src_gc, TRUE, 0, 0, width, height);
-
-    gdk_window_copy_area(out, src_gc, 0, 0, in, 0, 0, owidth, oheight);
-    g_object_unref(src);
-
-    return out;
-}
-
-GdkImage *create_dblsize_image(GdkImage * img)
-{
-    GdkImage *dblimg;
-    register guint x, y;
-
-    /*
-     * This needs to be optimized
-     */
-
-    dblimg =
-    gdk_image_new(GDK_IMAGE_NORMAL, gdk_visual_get_system(),
-              img->width << 1, img->height << 1);
-    if (dblimg->bpp == 1) {
-    register guint8 *srcptr, *ptr, *ptr2, pix;
-
-    srcptr = GDK_IMAGE(img)->mem;
-    ptr = GDK_IMAGE(dblimg)->mem;
-    ptr2 = ptr + dblimg->bpl;
-
-    for (y = 0; y < img->height; y++) {
-        for (x = 0; x < img->width; x++) {
-        pix = *srcptr++;
-        *ptr++ = pix;
-        *ptr++ = pix;
-        *ptr2++ = pix;
-        *ptr2++ = pix;
-        }
-        srcptr += img->bpl - img->width;
-        ptr += (dblimg->bpl << 1) - dblimg->width;
-        ptr2 += (dblimg->bpl << 1) - dblimg->width;
+    if (scale) {
+        GdkPixbuf *image = gdk_pixbuf_scale_simple(obj, width * cfg.scale_factor, height* cfg.scale_factor, GDK_INTERP_BILINEAR);
+        gdk_draw_pixbuf(widget->window, NULL, image, 0, 0, 0, 0, width * cfg.scale_factor , height * cfg.scale_factor, GDK_RGB_DITHER_NORMAL, 0, 0);
+        g_object_unref(image);
+    } else {
+        gdk_draw_pixbuf(widget->window, NULL, obj, 0, 0, 0, 0, width, height, GDK_RGB_DITHER_NONE, 0, 0);
     }
-    }
-    if (dblimg->bpp == 2) {
-    guint16 *srcptr, *ptr, *ptr2, pix;
-
-    srcptr = (guint16 *) GDK_IMAGE_XIMAGE(img)->data;
-    ptr = (guint16 *) GDK_IMAGE_XIMAGE(dblimg)->data;
-    ptr2 = ptr + (dblimg->bpl >> 1);
-
-    for (y = 0; y < img->height; y++) {
-        for (x = 0; x < img->width; x++) {
-        pix = *srcptr++;
-        *ptr++ = pix;
-        *ptr++ = pix;
-        *ptr2++ = pix;
-        *ptr2++ = pix;
-        }
-        srcptr += (img->bpl >> 1) - img->width;
-        ptr += (dblimg->bpl) - dblimg->width;
-        ptr2 += (dblimg->bpl) - dblimg->width;
-    }
-    }
-    if (dblimg->bpp == 3) {
-    register guint8 *srcptr, *ptr, *ptr2, pix1, pix2, pix3;
-
-    srcptr = GDK_IMAGE(img)->mem;
-    ptr = GDK_IMAGE(dblimg)->mem;
-    ptr2 = ptr + dblimg->bpl;
-
-    for (y = 0; y < img->height; y++) {
-        for (x = 0; x < img->width; x++) {
-        pix1 = *srcptr++;
-        pix2 = *srcptr++;
-        pix3 = *srcptr++;
-        *ptr++ = pix1;
-        *ptr++ = pix2;
-        *ptr++ = pix3;
-        *ptr++ = pix1;
-        *ptr++ = pix2;
-        *ptr++ = pix3;
-        *ptr2++ = pix1;
-        *ptr2++ = pix2;
-        *ptr2++ = pix3;
-        *ptr2++ = pix1;
-        *ptr2++ = pix2;
-        *ptr2++ = pix3;
-
-        }
-        srcptr += img->bpl - (img->width * 3);
-        ptr += (dblimg->bpl << 1) - (dblimg->width * 3);
-        ptr2 += (dblimg->bpl << 1) - (dblimg->width * 3);
-    }
-    }
-    if (dblimg->bpp == 4) {
-    register guint32 *srcptr, *ptr, *ptr2, pix;
-
-    srcptr = (guint32 *) GDK_IMAGE(img)->mem;
-    ptr = (guint32 *) GDK_IMAGE(dblimg)->mem;
-    ptr2 = ptr + (dblimg->bpl >> 2);
-
-    for (y = 0; y < img->height; y++) {
-        for (x = 0; x < img->width; x++) {
-        pix = *srcptr++;
-        *ptr++ = pix;
-        *ptr++ = pix;
-        *ptr2++ = pix;
-        *ptr2++ = pix;
-        }
-        srcptr += (img->bpl >> 2) - img->width;
-        ptr += (dblimg->bpl >> 1) - dblimg->width;
-        ptr2 += (dblimg->bpl >> 1) - dblimg->width;
-    }
-    }
-    return dblimg;
-}
-
-/* URL-decode a file: URL path, return NULL if it's not what we expect */
-gchar *
-xmms_urldecode_path(const gchar * encoded_path)
-{
-    const gchar *cur, *ext;
-    gchar *path, *tmp;
-    gint realchar;
-
-    if (!encoded_path)
-        return NULL;
-
-    if (!str_has_prefix_nocase(encoded_path, "file:"))
-        return NULL;
-
-    cur = encoded_path + 5;
-
-    if (str_has_prefix_nocase(cur, "//localhost"))
-        cur += 11;
-
-    if (*cur == '/')
-        while (cur[1] == '/')
-            cur++;
-
-    tmp = g_malloc0(strlen(cur) + 1);
-
-    while ((ext = strchr(cur, '%')) != NULL) {
-        strncat(tmp, cur, ext - cur);
-        ext++;
-        cur = ext + 2;
-        if (!sscanf(ext, "%2x", &realchar)) {
-            /* Assume it is a literal '%'.  Several file
-             * managers send unencoded file: urls on drag
-             * and drop. */
-            realchar = '%';
-            cur -= 2;
-        }
-        tmp[strlen(tmp)] = realchar;
-    }
-
-    path = g_strconcat(tmp, cur, NULL);
-    g_free(tmp);
-    return path;
 }
 
 /**
@@ -992,9 +937,9 @@ xmms_urldecode_path(const gchar * encoded_path)
  * Return value: A GTK widget handle for the message box.
  **/
 GtkWidget *
-xmms_show_message(const gchar * title, const gchar * text,
-                  const gchar * button_text, gboolean modal,
-                  GtkSignalFunc button_action, gpointer action_data)
+util_info_dialog(const gchar * title, const gchar * text,
+                 const gchar * button_text, gboolean modal,
+                 GCallback button_action, gpointer action_data)
 {
   GtkWidget *dialog;
   GtkWidget *dialog_vbox, *dialog_hbox, *dialog_bbox;
@@ -1031,13 +976,15 @@ xmms_show_message(const gchar * title, const gchar * text,
   if ( button_action )
     g_signal_connect( G_OBJECT(dialog_bbox_b1) , "clicked" ,
                       button_action , action_data );
-  GTK_WIDGET_SET_FLAGS( dialog_bbox_b1 , GTK_CAN_DEFAULT);
-  gtk_widget_grab_default( dialog_bbox_b1 );
 
   gtk_container_add( GTK_CONTAINER(dialog_bbox) , dialog_bbox_b1 );
   gtk_box_pack_start( GTK_BOX(dialog_vbox) , dialog_bbox , FALSE , FALSE , 0 );
 
   gtk_container_add( GTK_CONTAINER(dialog) , dialog_vbox );
+
+  GTK_WIDGET_SET_FLAGS( dialog_bbox_b1 , GTK_CAN_DEFAULT);
+  gtk_widget_grab_default( dialog_bbox_b1 );
+
   gtk_widget_show_all(dialog);
 
   return dialog;
@@ -1045,7 +992,7 @@ xmms_show_message(const gchar * title, const gchar * text,
 
 
 /**
- * audacious_get_localdir:
+ * util_get_localdir:
  *
  * Returns a string with the full path of Audacious local datadir (where config files are placed).
  * It's useful in order to put in the right place custom config files for audacious plugins.
@@ -1053,7 +1000,7 @@ xmms_show_message(const gchar * title, const gchar * text,
  * Return value: a string with full path of Audacious local datadir (should be freed after use)
  **/
 gchar*
-audacious_get_localdir(void)
+util_get_localdir(void)
 {
   gchar *datadir;
   gchar *tmp;
@@ -1067,27 +1014,65 @@ audacious_get_localdir(void)
 }
 
 
-/**
- * xmms_check_realtime_priority:
- *
- * Legacy function included for compatibility with XMMS.
- *
- * Return value: FALSE
- **/
-gboolean
-xmms_check_realtime_priority(void)
+gchar *
+construct_uri(gchar *string, const gchar *playlist_name) // uri, path and anything else
 {
-    return FALSE;
+    gchar *filename = g_strdup(string);
+    gchar *tmp, *path;
+    gchar *uri = NULL;
+
+    /* try to translate dos path */
+    convert_dos_path(filename); /* in place replacement */
+
+    /* convert backslash to slash */
+    while ((tmp = strchr(filename, '\\')) != NULL)
+        *tmp = '/';
+
+    // make full path uri here
+    // case 1: filename is raw full path or uri
+    if (filename[0] == '/' || strstr(filename, "://")) {
+        uri = g_filename_to_uri(filename, NULL, NULL);
+        if(!uri) {
+            uri = g_strdup(filename);
+        }
+        g_free(filename);
+    }
+    // case 2: filename is not raw full path nor uri, playlist path is full path
+    // make full path by replacing last part of playlist path with filename. (using g_build_filename)
+    else if (playlist_name[0] == '/' || strstr(playlist_name, "://")) {
+        path = g_strdup(playlist_name);
+        tmp = strrchr(path, '/'); *tmp = '\0';
+        tmp = g_build_filename(path, filename, NULL);
+        g_free(path); g_free(filename);
+        uri = g_filename_to_uri(tmp, NULL, NULL);
+        g_free(tmp);
+    }
+    // case 3: filename is not raw full path nor uri, playlist path is not full path
+    // just abort.
+    else {
+        g_free(filename);
+        return NULL;
+    }
+
+    AUDDBG("uri=%s\n", uri);
+    return uri;
 }
 
-/**
- * xmms_usleep:
- * @usec: The amount of microseconds to sleep.
+/* 
+ * minimize number of realloc's:
+ *  - set N to nearest power of 2 not less then N
+ *  - double it
  *
- * Legacy function included for compatibility with XMMS.
- **/
-void
-xmms_usleep(gint usec)
+ *  -- asphyx
+ *
+ * XXX: what's so smart about this?? seems wasteful and silly. --nenolod
+ */
+gpointer
+smart_realloc(gpointer ptr, gsize *size)
 {
-    g_usleep(usec);
+    *size = (size_t)pow(2, ceil(log(*size) / log(2)) + 1);
+    if (ptr != NULL) free(ptr);
+    ptr = malloc(*size);
+    return ptr;
 }
+
